package nl.kallestruik.ditems

import nl.kallestruik.ditems.dsl.item.OnBreakBlock
import nl.kallestruik.ditems.dsl.item.OnInventoryClick
import nl.kallestruik.ditems.dsl.item.OnRightClick
import org.bukkit.block.Block
import org.bukkit.event.EventHandler
import org.bukkit.event.Listener
import org.bukkit.event.block.Action
import org.bukkit.event.block.BlockBreakEvent
import org.bukkit.event.inventory.InventoryClickEvent
import org.bukkit.event.player.PlayerInteractEvent
import java.util.*

class EventListener(
    private val plugin: DItems,
): Listener {

    @EventHandler
    fun onBreakBlock(event: BlockBreakEvent) {
        val item = event.player.inventory.itemInMainHand
        val customItem = ItemManager.getCustomItem(item) ?: return

        val lastTwoTargetBlocks: List<Block> = event.player.getLastTwoTargetBlocks(null, 7)
        if (lastTwoTargetBlocks.size != 2 || !lastTwoTargetBlocks[1].type.isOccluding) return

        val targetBlock: Block = lastTwoTargetBlocks[1]
        val adjacentBlock: Block = lastTwoTargetBlocks[0]

        val face = targetBlock.getFace(adjacentBlock) ?: return

        // Make sure we don't break blocks other than the one that we intended
        if (targetBlock != event.block) return

        OnBreakBlock(event, event.block.location, face, item, customItem, plugin).apply(customItem.onBreakBlock)
    }

    @EventHandler
    fun onInventoryInteract(event: InventoryClickEvent) {
        val cursor = event.cursor
        val clicked = event.currentItem

        val cursorCustomItem = cursor?.let { ItemManager.getCustomItem(it) }
        if (cursorCustomItem != null)
            OnInventoryClick(event, cursor, cursorCustomItem, clicked, plugin).apply(cursorCustomItem.onInventoryClick)

        val clickedCustomItem = clicked?.let { ItemManager.getCustomItem(it) }
        if (clickedCustomItem != null)
            OnInventoryClick(event, clicked, clickedCustomItem, cursor, plugin).apply(clickedCustomItem.onInventoryClick)
    }

    @EventHandler
    fun onRightClick(event: PlayerInteractEvent) {
        if (!(event.action == Action.RIGHT_CLICK_AIR || event.action == Action.RIGHT_CLICK_BLOCK)) return

        val item = event.player.inventory.getItem(event.hand ?: return)

        val customItem = ItemManager.getCustomItem(item) ?: return

        if (isOnCooldown(event.player.uniqueId, customItem)) return

        OnRightClick(event, event.player, item, customItem, plugin).apply(customItem.onRightClick)
    }

    fun isOnCooldown(uuid: UUID, item: Item): Boolean {
        val cooldown = ItemManager.itemCooldowns[Pair(uuid, item.id)] ?: return false
        return System.currentTimeMillis() - cooldown.start < cooldown.length
    }
}