package nl.kallestruik.ditems

import net.kyori.adventure.text.Component
import net.kyori.adventure.text.format.TextDecoration
import nl.kallestruik.ditems.dsl.item.ItemAttribute
import nl.kallestruik.ditems.dsl.item.OnBreakBlock
import nl.kallestruik.ditems.dsl.item.OnInventoryClick
import nl.kallestruik.ditems.dsl.item.OnRightClick
import nl.kallestruik.ditems.dsl.exceptions.InvalidItemException
import org.bukkit.Material
import org.bukkit.attribute.AttributeModifier
import org.bukkit.inventory.ItemStack
import org.bukkit.persistence.PersistentDataType
import java.util.*

data class Item(
    val id: String,
    val name: Component,
    val baseItem: Material,
    val onBreakBlock: OnBreakBlock.() -> Unit = {},
    val onInventoryClick: OnInventoryClick.() -> Unit = {},
    val onRightClick: OnRightClick.() -> Unit = {},
    val finalizeItemStack: ItemStack.() -> Unit = {},
    val attributes: List<ItemAttribute> = emptyList(),
) {
    fun createItemStack(): ItemStack {
        val stack = ItemStack(baseItem)
        stack.editMeta {
            it.persistentDataContainer.set(ItemManager.ITEM_ID_KEY, PersistentDataType.STRING, id)

            it.displayName(Component.empty().decoration(TextDecoration.ITALIC, false).append(name))

            it.attributeModifiers = null

            for (attribute in attributes) {
                if (attribute.attribute == null)
                    throw InvalidItemException("Attributes need to at least have a attribute defined!")

                it.addAttributeModifier(attribute.attribute!!, AttributeModifier(UUID.randomUUID(), attribute.name, attribute.value, attribute.operation, attribute.slot))
            }
        }

        return stack.apply(finalizeItemStack)
    }
}
