package nl.kallestruik.ditems

import net.kyori.adventure.text.Component
import net.kyori.adventure.text.format.NamedTextColor
import net.kyori.adventure.text.format.TextColor
import nl.kallestruik.ditems.script.DItemsScriptHost
import org.bukkit.Bukkit
import org.bukkit.NamespacedKey
import org.bukkit.command.CommandSender
import org.bukkit.inventory.ItemStack
import org.bukkit.persistence.PersistentDataType
import java.io.File
import java.util.*
import kotlin.script.experimental.api.ScriptDiagnostic

object ItemManager {
    val items: MutableMap<String, Item> = mutableMapOf()
    val itemCooldowns: MutableMap<Pair<UUID, String>, ItemCooldown> = mutableMapOf()

    private lateinit var scriptFolder: File
    lateinit var ITEM_ID_KEY: NamespacedKey

    fun setup(plugin: DItems) {
        this.scriptFolder = File(plugin.dataFolder, "items")
        this.scriptFolder.mkdirs()

        ITEM_ID_KEY = NamespacedKey(plugin, "custom_item_id")
    }

    private fun colorForSeverity(severity: ScriptDiagnostic.Severity): TextColor {
        return when (severity) {
            ScriptDiagnostic.Severity.DEBUG -> NamedTextColor.GRAY
            ScriptDiagnostic.Severity.INFO -> NamedTextColor.BLUE
            ScriptDiagnostic.Severity.WARNING -> NamedTextColor.YELLOW
            ScriptDiagnostic.Severity.ERROR -> NamedTextColor.RED
            ScriptDiagnostic.Severity.FATAL -> NamedTextColor.DARK_RED
        }
    }

    fun loadItems(logTarget: CommandSender = Bukkit.getServer().consoleSender) {
        val loadStartTime = System.nanoTime()

        val scriptFiles = scriptFolder
            .listFiles()
            ?.filter { it.isFile }
            ?.filter { it.name.endsWith(".ditems.kts") }
            ?: listOf()

        val reports = scriptFiles
            .flatMap { DItemsScriptHost.evalFile(it) }

        reports
            .filter { it.severity > ScriptDiagnostic.Severity.DEBUG }
            .forEach {
                logTarget.sendMessage(
                    Component.empty()
                        .append(Component.text("[${it.severity.name}] ").color(colorForSeverity(it.severity)))
                        .append(Component.text("(${it.sourcePath}: ${it.location?.start?.line}) "))
                        .append(Component.text(it.message))
                )
            }

        val loadTime = System.nanoTime() - loadStartTime
        val loadTimeString = "%.1f".format(loadTime / 1000000000.0)

        logTarget.sendMessage(Component.empty()
            .append(Component.text("[DItems] ").color(NamedTextColor.GOLD))
            .append(Component.empty().color(NamedTextColor.YELLOW)
                .append(Component.text("Loaded "))
                .append(Component.text(items.size).color(NamedTextColor.AQUA))
                .append(Component.text(" items in "))
                .append(Component.empty().color(NamedTextColor.AQUA)
                    .append(Component.text(loadTimeString))
                    .append(Component.text("s"))
                )
            )
        )
    }

    fun getCustomItem(stack: ItemStack): Item? {
        val meta = stack.itemMeta ?: return null
        val dataContainer = meta.persistentDataContainer
        if (!dataContainer.has(ITEM_ID_KEY))
            return null

        val id = dataContainer.get(ITEM_ID_KEY, PersistentDataType.STRING) ?: return null

        return items[id]
    }
}