package nl.kallestruik.ditems.dsl.item

import net.kyori.adventure.text.Component
import nl.kallestruik.ditems.Item
import nl.kallestruik.ditems.ItemManager
import nl.kallestruik.ditems.dsl.exceptions.InvalidItemException
import org.bukkit.Material
import org.bukkit.inventory.ItemStack

fun item(config: ItemBuilder.() -> Unit): Item {
    val item = ItemBuilder().apply(config).build()
    ItemManager.items[item.id] = item

    return item
}

class ItemBuilder {
    var id: String? = null
    var name: Component? = null
    var baseItem: Material = Material.AIR
    private var onBreakBlock: OnBreakBlock.() -> Unit = {}
    private var onInventoryClick: OnInventoryClick.() -> Unit = {}
    private var onRightClick: OnRightClick.() -> Unit = {}
    private var finalizeItemStack: ItemStack.() -> Unit = {}
    private val attributes = mutableListOf<ItemAttribute>()

    fun onBreakBlock(onBreak: OnBreakBlock.() -> Unit) {
        this.onBreakBlock = onBreak
    }

    fun onInventoryClick(onInventoryClick: OnInventoryClick.() -> Unit) {
        this.onInventoryClick = onInventoryClick
    }

    fun onRightClick(onRightClick: OnRightClick.() -> Unit) {
        this.onRightClick = onRightClick
    }

    fun finalizeItemStack(finalizeItemStack: ItemStack.() -> Unit) {
        this.finalizeItemStack = finalizeItemStack
    }

    fun attribute(itemAttribute: ItemAttribute.() -> Unit) {
        attributes += ItemAttribute().apply(itemAttribute)
    }

    fun build(): Item {
        return Item(
            id = id ?: throw InvalidItemException("An item needs to have an id!"),
            name = name  ?: throw InvalidItemException("An item needs to have a name!"),
            baseItem = baseItem,
            onBreakBlock = onBreakBlock,
            onInventoryClick = onInventoryClick,
            onRightClick = onRightClick,
            finalizeItemStack = finalizeItemStack,
            attributes = attributes,
        )
    }
}