package nl.kallestruik.ditems.dsl.item

import nl.kallestruik.ditems.Item
import nl.kallestruik.ditems.ItemCooldown
import nl.kallestruik.ditems.ItemManager
import org.bukkit.Location
import org.bukkit.entity.*
import org.bukkit.event.player.PlayerInteractEvent
import org.bukkit.inventory.ItemStack
import org.bukkit.plugin.java.JavaPlugin

class OnRightClick(
    val event: PlayerInteractEvent,
    val player: Player,
    val itemStack: ItemStack,
    val item: Item,
    val plugin: JavaPlugin,
) {
    fun summon(where: Location, type: EntityType): Entity {
        return player.world.spawn(where, type.entityClass!!)
    }

    fun shoot(where: Location, type: EntityType, speed: Double): Projectile {
        val projectile = summon(where, type) as Projectile
        projectile.velocity = player.location.direction.clone().normalize().multiply(speed)
        projectile.shooter = player
        return projectile
    }

    fun fireball(where: Location, type: EntityType): Fireball {
        val fireball = shoot(where, type, 0.0) as Fireball
        fireball.direction = player.location.direction.clone().normalize()
        return fireball
    }

    fun relative(x: Double, y: Double, z: Double): Location {
        return player.location.relative(x, y, z)
    }

    fun forward(amount: Double): Location {
        return player.eyeLocation.forward(amount)
    }

    private fun Location.relative(x: Double, y: Double, z: Double): Location {
        return this.clone().add(x, y, z)
    }

    private fun Location.forward(amount: Double): Location {
        return this.clone().add(player.location.direction.clone().normalize().multiply(amount))
    }

    fun cooldown(ticks: Int) {
        player.setCooldown(item.baseItem, ticks)
        ItemManager.itemCooldowns[Pair(player.uniqueId, item.id)] = ItemCooldown(System.currentTimeMillis(), ticks * 50L)
    }
}